//
//  DRScreenRecorder.h
//  DeepRecorder
//
//  Created by mmivi on 20.04.09.
//  Copyright 2009 DeepIT. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "DRSnap.h"
#import "DRMovieFrame.h"
#import "DRCursor.h"

#define DR_SCRREC_DESIRED_FPS 60.0

@interface DRScreenRecorder : NSObject {
	NSRect _captureRect;
	CFAbsoluteTime _captureDuration;
	long _capturedFrames;
	float _desiredFPS;
	
	CVDisplayLinkRef _displayLink;
	DRSnap * _snapper;
	
	BOOL _keepRunning, _stopNow, _pause;
	
	id _delegate, _dataDestination;
	
	CFAbsoluteTime startTime, pauseTime, prevPauseTime;
	NSMutableArray * _cursors;
	
	int currentCursorIndex;
	int prevCursorSeed;
	BOOL isPrevCursorSeedInitialized;
	
	CFAbsoluteTime _realStartTime, _startOffsetTime;
	
	BOOL _suspend;
	BOOL _zlibCompression;
}

- (CVReturn)displayLinkCallback:(const CVTimeStamp*)timeStamp flagsOut:(CVOptionFlags*)flagsOut;

- (id) initWithRect: (NSRect)rect;
- (void)startSetRect;
- (void)setRect: (NSRect)rect;
- (void)endSetRect;

- (BOOL)isSuspended;
- (void)setSuspended:(BOOL)flag;

- (void)startAtTime: (CFAbsoluteTime)theTime;
- (CFAbsoluteTime)startOffsetTime;
- (void)start;
- (void)stop;
- (BOOL)isRunning;
- (void)pause: (BOOL)flag;

- (void)setDataDestination: (id)object;
- (void)setDelegate: (id)object;

- (CFAbsoluteTime)captureDuration;
- (long)capturedFrames;
- (NSArray *)capturedCursors;

- (void)setZlibCompression:(BOOL)flag;
- (BOOL)isZlibCompressionEnabled;

@end

@interface NSObject (DRScreenRecorderDelegateInformalProtocol)

- (void)DRScreenRecorderCapturedFrame: (DRScreenRecorder *)recorder;
- (void)DRScreenRecorderDidStop: (DRScreenRecorder *)recorder;
- (void)DRScreenRecorderDidStart: (DRScreenRecorder *)recorder;
- (void)DRScreenRecorderWillStart: (DRScreenRecorder *)recorder;

@end
