//
//  DRWavIO.h
//  DeepRecorder
//
//  Created by mmivi on 26.03.09.
//  Copyright 2009 DeepIT. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "DRSoundPacket.h"
#import "types.h"

#define DR_WAVIO_PAUSE_KEY @"PauseStateKey"

//typedef uint32_t DWORD;
//typedef uint16_t WORD;

typedef struct
	{
		DWORD ChunkID;
		DWORD ChunkSize;
		DWORD Format;
	} RIFF_Header;

typedef struct 
	{
		DWORD Subchunk1ID;
		DWORD Subchunk1Size;
		WORD AudioFormat;
		WORD NumChannels;
		DWORD SampleRate;
		DWORD ByteRate;
		WORD BlockAlign;
		WORD BitsPerSample;
		//---
		WORD cbSize;
		//---
		WORD wValidBitsPerSample;
		DWORD dwChannelMask;
		unsigned char SubFormat[16];
	} WAVE_Format_Chunk;

typedef struct
	{
		DWORD Subchunk2ID;
		DWORD Subchunk2Size;
	} WAVE_Data_Chunk;

typedef  struct {
	DWORD ckID;
	DWORD cksize;
	DWORD dwSampleLength;
} WAVE_Fact_Chunk;

#define BIG32(arg) CFSwapInt32HostToBig(arg)
#define LITTLE32(arg) CFSwapInt32HostToLittle(arg)
#define BIG16(arg) CFSwapInt16HostToBig(arg)
#define LITTLE16(arg) CFSwapInt16HostToLittle(arg)

#define WAVE_FORMAT_PCM 0x0001
#define WAVE_FORMAT_IEEE_FLOAT 0x0003
#define WAVE_FORMAT_ALAW 0x0006
#define WAVE_FORMAT_MULAW 0x0007
#define WAVE_FORMAT_EXTENSIBLE 0xFFFE

typedef struct
	{
		RIFF_Header head;
		WAVE_Format_Chunk ch1;
		WAVE_Data_Chunk ch2;
	} WavFileInfo;

typedef enum
	{
		DRWavIOReadMode,
		DRWavIOWriteMode
	} DRWavIOMode;
	
@interface DRWavIO : NSObject {
	
	RIFF_Header head;
	WAVE_Format_Chunk chFormat;
	WAVE_Data_Chunk chData;
	WAVE_Fact_Chunk chFact;
	
	NSString * _fullPath;
	DRWavIOMode _mode;
	DRSoundFormat _format;
	int _bytesPerFrame;
	
	unsigned int _framesCount;
	int _chunkSize;
	
	NSFileHandle * _wavFile;
	
	id _delegate;
	id _dataSource;
	id _dataDestination;
	
	BOOL _keepRunning, _stopNow;
	BOOL _isPaused;
	BOOL _finalize;
}

+ (id)wavIOForWriting: (NSString *)fullPath format: (DRSoundFormat)format;
+ (id)wavIOForReading: (NSString *)fullPath;
- (id) initWithFile: (NSString *)fullPath mode: (DRWavIOMode)mode;

- (void)start;
- (void)stop;
- (void)finalize;
- (void)pause: (BOOL)flag;

- (void)setDelegate: (id)object;
- (id)dataSource;
- (void)setDataSource: (id)object;
- (void)setDataDestination: (id)object;
- (id)dataDestination;

- (void)setFormat: (DRSoundFormat)format;
- (DRSoundFormat)format;

- (WavFileInfo)fileInfo;
- (long)frames;
- (CFAbsoluteTime)duration;

@end

@interface NSObject (DRWavIODelegateInformalProtocol)

- (void)DRWavIODidStart: (DRWavIO *)wavIO;
- (void)DRWavIODidStop: (DRWavIO *)wavIO;
- (void)DRWavIO: (DRWavIO *)wavIO didChangePauseState: (BOOL)flag;

@end


@interface NSObject (DRWavIODataSourceInformalProtocol)

- (DRSoundPacket *)nextObject;

@end

@interface NSObject (DRWavIODataDestinationInformalProtocol)

- (void)addObject: (DRSoundPacket *)object;
- (BOOL)canAcceptObject;

@end

